#!/usr/bin/python3
# encoding: utf-8

# --                                                            ; {{{1
#
# File        : kanjidraw/gui.py
# Maintainer  : Felix C. Stegerman <flx@obfusk.net>
# Date        : 2021-05-19
#
# Copyright   : Copyright (C) 2021  Felix C. Stegerman
# Version     : v0.2.2
# License     : AGPLv3+
#
# --                                                            ; }}}1

                                                                # {{{1
r"""

Handwritten kanji recognition: tkinter GUI.

"""                                                             # }}}1

import argparse, os, sys

import tkinter as tk
import tkinter.font

from tkinter import ttk

from .lib import kanji_data, matches, __version__

NAME, TITLE = "kanjidraw", "Kanji Draw"
HEIGHT = WIDTH = 400
COLS, LINEWIDTH, FONTSIZE = 5, 5, 35
FONTS = ("Noto Sans CJK JP", "Noto Sans CJK SC", "Noto Sans CJK TC",
         "IPAexGothic", "IPAGothic")

LIGHT_THEME = dict(
  bg = "#fff", bg_active = "#eee", fg = "#000", btn_fg = "#fff",
  btn_bg = "#375a7f", btn_bg_active = "#1a252f",
  btn_bg_disabled = "#7c93ab", btn_fg_disabled = "#fff",
  canvas_fg = "#000", grid = "#999"
)
DARK_THEME = dict(
  bg = "#333", bg_active = "#222", fg = "#fff", btn_fg = "#fff",
  btn_bg = "#375a7f", btn_bg_active = "#28415b",
  btn_bg_disabled = "#344b63", btn_fg_disabled = "#b7b7b7",
  canvas_fg = "#ddd", grid = "#999"
)

def gui(stdout = False, oneshot = False, multiple = False,      # {{{1
        dark = False):
  """Tkinter GUI."""

  nogrid = os.environ.get("KANJIDRAW_NOGRID") in ("1", "true", "yes")
  if os.environ.get("KANJIDRAW_DARK") in ("1", "true", "yes"):
    dark = True

  win = tk.Tk()
  win.title(TITLE)
  win.columnconfigure(0, weight = 1)
  win.rowconfigure(0, weight = 1)

  theme = DARK_THEME if dark else LIGHT_THEME
  fonts = set(tkinter.font.families())
  kanji_font = tk.font.Font()

  for f in FONTS:
    if f in fonts:
      kanji_font.config(family = f)
      break

  kanji_btn_font = kanji_font.copy()
  kanji_btn_font.config(size = FONTSIZE)

  s = ttk.Style()
  s.configure(".", background = theme["bg"], foreground = theme["fg"])
  s.configure("TButton", background = theme["btn_bg"],
                         foreground = theme["btn_fg"])
  s.map(".", background = [("active", theme["bg_active"]),
                           ("disabled", theme["bg"])])
  s.map("TButton", background = [("active", theme["btn_bg_active"]),
                                 ("disabled", theme["btn_bg_disabled"])],
                   foreground = [("active", theme["btn_fg"]),
                                 ("disabled", theme["btn_fg_disabled"])])
  s.configure("Kanji.TButton", font = kanji_btn_font, width = 0)
  s.configure("Kanji.TLabel", font = kanji_font)

  max_strokes = max(kanji_data().keys())
  drawing, x, y, strokes, lines = False, 0, 0, [], []

  def on_mousedown(event):
    nonlocal drawing, x, y
    if len(strokes) < max_strokes:
      drawing, x, y = True, event.x, event.y
      strokes.append([x * 255.0 / WIDTH, y * 255.0 / HEIGHT])
      lines.append([])
      enable_buttons()

  def on_mousemove(event):
    if drawing: draw_line(event.x, event.y)

  def on_mouseup(event):
    nonlocal drawing
    if drawing:
      draw_line(event.x, event.y)
      drawing = False
      strokes[-1] += [x * 255.0 / WIDTH, y * 255.0 / HEIGHT]
      update_strokes()

  def on_undo():
    if strokes:
      strokes.pop()
      for l in lines.pop(): canvas.delete(l)
      update_strokes()
      if not strokes: disable_buttons()

  def on_clear():
    strokes.clear(); lines.clear()
    canvas.delete("all"); draw_grid()
    update_strokes(); disable_buttons()

  def on_done():
    res_frame = ttk.Frame(win)
    res_btns  = ttk.Frame(res_frame)
    btn_back  = ttk.Button(res_btns, text = "Go Back", command = on_back(res_frame))
    lbl_info  = ttk.Label(res_btns, text = info_txt)
    res_grid  = ttk.Frame(res_frame)

    ms = matches(strokes, fuzzy = var_fuzzy.get(), offby1 = var_ob1.get())
    for i, (_, kanji) in enumerate(ms):
      col, row = i % COLS, i // COLS
      res_grid.columnconfigure(col, weight = 1)
      res_grid.rowconfigure(row, weight = 1)
      btn = ttk.Button(res_grid, text = " " + kanji + " ", style = "Kanji.TButton",
                       command = on_select_kanji(res_frame, kanji))
      btn.grid(column = col, row = row, sticky = "nsew")

    btn_back.pack(side = tk.LEFT, padx = 5, pady = 5)
    lbl_info.pack(side = tk.LEFT, padx = 5, pady = 5)
    res_btns.pack(); res_grid.pack()
    res_frame.grid(row = 0, column = 0, sticky = "nsew")
    win.bind("<Escape>", on_back(res_frame))

  def on_back(res_frame):
    def f(event = None):
      win.unbind("<Escape>")
      res_frame.destroy()
    return f

  def on_select_kanji(res_frame, kanji):
    def f():
      nonlocal drawn_kanji
      if multiple:
        drawn_kanji += kanji
        kanji_lbl.config(text = drawn_kanji)
      else:
        (print if stdout else copy_to_clipboard)(kanji)
        if oneshot: win.quit()
      res_frame.destroy()
      on_clear()
    return f

  def draw_line(x2, y2):
    nonlocal x, y
    l = canvas.create_line(x, y, x2, y2, width = LINEWIDTH,
                           capstyle = tk.ROUND, fill = theme["canvas_fg"])
    lines[-1].append(l)
    x, y = x2, y2

  def draw_grid():
    if not nogrid:
      for x in (WIDTH // 3, 2 * WIDTH // 3):
        canvas.create_line(x, 0, x, HEIGHT, fill = theme["grid"])
      for y in (HEIGHT // 3, 2 * HEIGHT // 3):
        canvas.create_line(0, y, WIDTH, y, fill = theme["grid"])

  def disable_buttons():
    for w in [btn_undo, btn_clear, btn_done]: w.config(state = tk.DISABLED)

  def enable_buttons():
    for w in [btn_undo, btn_clear, btn_done]: w.config(state = tk.NORMAL)

  def update_strokes():
    lbl_strokes.config(text = "Strokes: {}".format(len(strokes)))

  def copy_to_clipboard(text):
    win.clipboard_clear()
    win.clipboard_append(text)

  draw_frame  = ttk.Frame(win)
  btns        = ttk.Frame(draw_frame)
  btn_undo    = ttk.Button(btns, text = "Undo", command = on_undo)
  btn_clear   = ttk.Button(btns, text = "Clear", command = on_clear)
  lbl_strokes = ttk.Label(btns, text = "Strokes: 0")
  btn_done    = ttk.Button(btns, text = "Done", command = on_done)
  checks      = ttk.Frame(draw_frame)
  var_fuzzy   = tk.IntVar()
  var_ob1     = tk.IntVar()
  check_fuzzy = ttk.Checkbutton(checks, variable = var_fuzzy,
                                text = "Ignore stroke order & direction")
  check_ob1   = ttk.Checkbutton(checks, variable = var_ob1, text = "± 1 stroke")

  canvas = tk.Canvas(draw_frame, height = HEIGHT, width = WIDTH,
                     background = theme["bg"], highlightthickness = 1,
                     highlightbackground = theme["canvas_fg"])
  canvas.bind("<ButtonPress-1>", on_mousedown)
  canvas.bind("<B1-Motion>", on_mousemove)
  canvas.bind("<ButtonRelease-1>", on_mouseup)

  draw_grid(); disable_buttons()
  for w in [btn_undo, btn_clear, lbl_strokes, btn_done, check_fuzzy, check_ob1]:
    w.pack(side = tk.LEFT, padx = 5, pady = 5)
  btns.pack(); checks.pack(); canvas.pack()
  draw_frame.grid(row = 0, column = 0, sticky = "nsew")

  if multiple:
    info_txt    = "Click to add to queue"                     #  FIXME
    drawn_kanji = ""
    kanji_lbl   = ttk.Label(draw_frame, style = "Kanji.TLabel")
    kanji_lbl.pack()

    def output():
      nonlocal drawn_kanji
      if drawn_kanji:
        (print if stdout else copy_to_clipboard)(drawn_kanji)
        drawn_kanji = ""
        kanji_lbl.config(text = "")

    def quit():
      output()
      win.quit()

    win.bind("c", lambda e: output())
    win.bind("q", lambda e: quit())
    win.protocol("WM_DELETE_WINDOW", quit)
  else:
    if stdout:
      info_txt = "Click to print to stdout"
    else:
      info_txt = "Click to copy to clipboard"
    win.bind("q", lambda e: win.quit())

  win.mainloop()
                                                                # }}}1

def main():                                                     # {{{1
  p = argparse.ArgumentParser(prog = NAME)
  p.add_argument("-s", "--stdout", action = "store_true",
                 help = "print kanji to stdout instead of "
                        "copying to clipboard")
  g = p.add_mutually_exclusive_group()
  g.add_argument("-o", "--oneshot", action = "store_true",
                 help = "quit after one kanji")
  g.add_argument("-m", "--multiple", action = "store_true",
                 help = "queue kanji and copy/print after "
                        "pressing 'c' or quitting")
  p.add_argument("-d", "--dark", action = "store_true",
                 help = "use dark theme")
  p.add_argument("--version", action = "version",
                 version = "%(prog)s {}".format(__version__))
  gui(**vars(p.parse_args()))
                                                                # }}}1

if __name__ == "__main__":
  if "--doctest" in sys.argv:
    verbose = "--verbose" in sys.argv
    import doctest
    if doctest.testmod(verbose = verbose)[0]: sys.exit(1)
  else:
    main()

# vim: set tw=70 sw=2 sts=2 et fdm=marker :



#!/usr/bin/python3
# encoding: utf-8

# --                                                            ; {{{1
#
# File        : kanjidraw/lib.py
# Maintainer  : Felix C. Stegerman <flx@obfusk.net>
# Date        : 2021-05-19
#
# Copyright   : Copyright (C) 2021  Felix C. Stegerman
# Version     : v0.2.2
# License     : AGPLv3+
#
# --                                                            ; }}}1

                                                                # {{{1
r"""

Handwritten kanji recognition: library.

>>> strokes = [[125.5875, 28.6875, 48.45, 196.35], [104.55, 93.7125, 195.7125, 223.125]]
>>> for s, k in matches(strokes): print(int(s), k)
99 人
96 九
96 乂
93 八
93 入
90 儿
89 勹
88 乃
87 又
87 几
87 冂
85 匕
82 亻
81 卜
78 亠
75 冖

>>> strokes = [[17.85, 102.0, 83.5125, 51.0], [45.9, 26.775, 49.0875, 210.375], [33.7875, 152.3625, 61.8375, 133.875], [103.9125, 54.825, 211.0125, 58.65], [139.6125, 31.2375, 142.8, 64.3875], [178.5, 42.075, 179.1375, 66.3], [108.375, 121.125, 106.4625, 182.325], [113.475, 114.1125, 205.275, 189.975], [163.2, 116.025, 164.475, 181.05], [126.225, 148.5375, 198.2625, 158.7375], [109.0125, 200.8125, 205.9125, 191.25]]
>>> for s, k in list(matches(strokes))[:5]: print(int(s), k)
91 描
89 猫
86 桷
85 淌
84 猟

>>> strokes = [[15.9375, 95.625, 30.6, 165.1125], [131.325, 32.5125, 128.1375, 222.4875], [208.4625, 63.1125, 205.9125, 219.9375]]
>>> for s, k in list(fuzzy_matches_offby1(strokes))[:5]: print(int(s), k)
91 水
91 刈
89 仆
89 爪
89 双

>>> strokes = [[26.1375, 94.35, 8.2875, 171.4875], [127.5, 24.8625, 123.0375, 211.0125], [214.2, 61.2, 135.15, 121.125], [143.4375, 137.7, 224.4, 230.1375]]
>>> for s, k in list(fuzzy_matches(strokes))[:5]: print(int(s), k)
99 水
88 尺
87 月
82 灬
82 曰

>>> strokes = [[123.675, 31.875, 102.6375, 221.85], [13.3875, 114.1125, 44.625, 177.8625], [236.5125, 63.1125, 221.2125, 219.3]]
>>> for s, k in list(strict_matches_offby1(strokes))[:5]: print(int(s), k)
90 水
89 刈
87 卜
87 凵
87 儿

"""                                                             # }}}1

import gzip, functools, itertools, json, os, re, sys
import xml.etree.ElementTree as ET

from collections import namedtuple
from enum import Enum

__version__ = "0.2.2"

DATAFILE = os.path.join(os.path.dirname(__file__), "data.json")

ARGS_RX = re.compile(r"(-?(?:\d*\.)?\d+)[\s,]*")
PATH_RX = re.compile(r"([MZCS])\s*((?:(?:-?(?:\d*\.)?\d+)[\s,]*)*)", re.I)

DIRECTION_THRESHOLD     = 51
DIAGONAL_THRESHOLD      = 77

STROKE_DIRECTION_WEIGHT = 1.0
MOVE_DIRECTION_WEIGHT   = 0.8
STROKE_LOCATION_WEIGHT  = 0.6
CLOSE_WEIGHT            = 0.7

MAX_RESULTS             = 25
CUTOFF                  = 0.75

class Direction(Enum):                                          # {{{1
  X, N, NE, E, SE, S, SW, W, NW = range(-1, 8)

  @classmethod
  def of_line(cls, l, threshold = DIRECTION_THRESHOLD):
    x1, y1, x2, y2 = l
    dx, dy = x2 - x1, y2 - y1
    adx, ady = abs(dx), abs(dy)
    if adx < threshold and ady < threshold:
      return cls.X
    if adx > ady:
      diag = ady > DIAGONAL_THRESHOLD * adx // 256
      if dx > 0:
        return (cls.NE if dy < 0 else cls.SE) if diag else cls.E
      else:
        return (cls.NW if dy < 0 else cls.SW) if diag else cls.W
    else:
      diag = adx > DIAGONAL_THRESHOLD * ady // 256
      if dy > 0:
        return (cls.SW if dx < 0 else cls.SE) if diag else cls.S
      else:
        return (cls.NW if dx < 0 else cls.NE) if diag else cls.N

  @classmethod
  def of_move(cls, l1, l2, threshold = DIRECTION_THRESHOLD):
    return cls.of_line(tuple(l1[2:] + l2[:2]), threshold)

  def isclose(a, b):
    return (a == a.X or b == a.X or a == b) or \
           (a.value == ((b.value+1)%8) or ((a.value+1)%8) == b.value)
                                                                # }}}1

@functools.total_ordering
class Location(Enum):                                           # {{{1
  N   = (1, 0)
  NE  = (2, 0)
  E   = (2, 1)
  SE  = (2, 2)
  S   = (1, 2)
  SW  = (0, 2)
  W   = (0, 1)
  NW  = (0, 0)
  MID = (1, 1)

  @classmethod
  def of_point(cls, x, y):
    if x < 85:
      return cls.NW if y < 85 else (cls.W if y < 170 else cls.SW)
    elif x < 170:
      return cls.N if y < 85 else (cls.MID if y < 170 else cls.S)
    else:
      return cls.NE if y < 85 else (cls.E if y < 170 else cls.SE)

  def isclose(a, b):
    return abs(a.value[0] - b.value[0]) <= 1 and \
           abs(a.value[1] - b.value[1]) <= 1

  def __lt__(a, b):
    return a.value < b.value

  def __eq__(a, b):
    return a.value == b.value
                                                                # }}}1

class Kanji(tuple):                                             # {{{1
  def __new__(cls, lines, fuzzy = False):
    if fuzzy: lines = _fuzzy_sort(lines)
    return super().__new__(cls, lines)

  def __init__(self, lines, fuzzy = False):
    self._fuzzy = self if fuzzy else None
    self._starts = self._ends = self._dirs = self._moves = None

  @property
  def fuzzy(self):
    if self._fuzzy is None:
      self._fuzzy = self.__class__(self, True)
    return self._fuzzy

  @property
  def starts(self):
    if self._starts is None:
      self._starts = tuple( Location.of_point(*l[:2]) for l in self )
    return self._starts

  @property
  def ends(self):
    if self._ends is None:
      self._ends = tuple( Location.of_point(*l[2:]) for l in self )
    return self._ends

  @property
  def dirs(self):
    if self._dirs is None:
      self._dirs = tuple(map(Direction.of_line, self))
    return self._dirs

  @property
  def moves(self):
    if self._moves is None:
      self._moves = tuple(map(Direction.of_move, self[1:], self[:-1]))
    return self._moves

  def minus_1_stroke(self):
    fuzzy = self._fuzzy is self
    for i in range(len(self)):
      x = self.__class__(self[:i] + self[i+1:])
      if fuzzy: x._fuzzy = x
      yield x
                                                                # }}}1

def _fuzzy_sort(lines):                                         # {{{1
  result = []
  for line in lines:
    start_p, end_p = line[:2], line[2:]
    start_l, end_l = Location.of_point(*start_p), Location.of_point(*end_p)
    if start_l > end_l:
      start_p, end_p = end_p, start_p
      start_l, end_l = end_l, start_l
    result.append((start_l, end_l, start_p + end_p))
  return tuple( x[2] for x in sorted(result) )
                                                                # }}}1

def strict_match(a, b):                                         # {{{1
  """Strict comparison; returns a percentage score as a float."""
  if len(a) != len(b): raise ValueError("must have same length")
  if not isinstance(a, Kanji): a = Kanji(a)
  if not isinstance(b, Kanji): b = Kanji(b)
  score, l = 0.0, len(a)
  for i in range(l):
    if a.dirs[i] == b.dirs[i]:
      score += STROKE_DIRECTION_WEIGHT
    elif a.dirs[i].isclose(b.dirs[i]):
      score += STROKE_DIRECTION_WEIGHT * CLOSE_WEIGHT
    if i > 0:
      if a.moves[i-1] == b.moves[i-1]:
        score += MOVE_DIRECTION_WEIGHT
      elif a.moves[i-1].isclose(b.moves[i-1]):
        score += MOVE_DIRECTION_WEIGHT * CLOSE_WEIGHT
    if a.starts[i] == b.starts[i]:
      score += STROKE_LOCATION_WEIGHT
    elif a.starts[i].isclose(b.starts[i]):
      score += STROKE_LOCATION_WEIGHT * CLOSE_WEIGHT
    if a.ends[i] == b.ends[i]:
      score += STROKE_LOCATION_WEIGHT
    elif a.ends[i].isclose(b.ends[i]):
      score += STROKE_LOCATION_WEIGHT * CLOSE_WEIGHT
  m = l * (STROKE_DIRECTION_WEIGHT + 2 * STROKE_LOCATION_WEIGHT) \
    + (l-1) * MOVE_DIRECTION_WEIGHT
  return 100 * score / m
                                                                # }}}1

def fuzzy_match(a, b):
  """Fuzzy comparison; ignores order and direction of strokes."""
  if len(a) != len(b): raise ValueError("must have same length")
  if not isinstance(a, Kanji): a = Kanji(a)
  if not isinstance(b, Kanji): b = Kanji(b)
  return strict_match(a.fuzzy, b.fuzzy)

def strict_match_offby1(a, b):
  """Strict comparison ± 1 stroke."""
  if not isinstance(a, Kanji): a = Kanji(a)
  if not isinstance(b, Kanji): b = Kanji(b)
  return _match_offby1(a, b, strict_match)

def fuzzy_match_offby1(a, b):
  """Fuzzy comparison ± 1 stroke."""
  if not isinstance(a, Kanji): a = Kanji(a)
  if not isinstance(b, Kanji): b = Kanji(b)
  return _match_offby1(a.fuzzy, b.fuzzy, strict_match)

# FIXME: deprecated
def match_offby1(match):
  """Comparison ± 1 stroke; deprecated."""
  if not isinstance(a, Kanji): a = Kanji(a)
  if not isinstance(b, Kanji): b = Kanji(b)
  return lambda a, b: _match_offby1(a, b, match)

def _match_offby1(a, b, match):
  if abs(len(a) - len(b)) != 1: raise ValueError("length difference must be 1")
  if len(a) > len(b): a, b = b, a
  return max( match(a, c) for c in b.minus_1_stroke() )

def matches(lines, data = None, fuzzy = False, offby1 = False,
            max_results = MAX_RESULTS, cutoff = CUTOFF):
  """
  Find best matches; yields a (score, kanji) pair for the first
  max_results matches that have a score >= max_score * cutoff.
  """
  data_items = _data_items_offby1 if offby1 else _data_items
  if offby1:
    match = fuzzy_match_offby1 if fuzzy else strict_match_offby1
  else:
    match = fuzzy_match if fuzzy else strict_match
  if not isinstance(lines, Kanji): lines = Kanji(lines)
  if fuzzy: lines = lines.fuzzy
  return _matches(lines, data, max_results, cutoff, match, data_items)

def strict_matches(*a, **kw):
  """Find strict matches."""
  return matches(*a, **kw)

def fuzzy_matches(*a, **kw):
  """Find fuzzy matches."""
  return matches(*a, fuzzy = True, **kw)

def strict_matches_offby1(*a, **kw):
  """Find strict matches ِِ± 1 stroke."""
  return matches(*a, offby1 = True, **kw)

def fuzzy_matches_offby1(*a, **kw):
  """Find fuzzy matches ِِ± 1 stroke."""
  return matches(*a, fuzzy = True, offby1 = True, **kw)

def _data_items(lines, data):
  return data[len(lines)].items()

def _data_items_offby1(lines, data):
  for n in [len(lines)-1, len(lines)+1]:
    if n in data: yield from data[n].items()

def _matches(lines, data, max_results, cutoff, match, data_items):
  it = data_items(lines, data or kanji_data())
  ms = sorted(( (match(lines, l), k) for k, l in it ), reverse = True)
  mm = ms[0][0] * cutoff
  return itertools.takewhile(lambda m: m[0] >= mm, ms[:max_results])

def kanji_data():
  if kanji_data._data is None: kanji_data._data = _load_json()
  return kanji_data._data
kanji_data._data = None

# FIXME: better kanji unicode ranges?
def _parse_kanjivg(file):                                       # {{{1
  data = {}
  with gzip.open(file) as f:
    for e in ET.parse(f).getroot():
      if e.tag != "kanji": continue
      code  = int(e.get("id").replace("kvg:kanji_", ""), 16)
      char  = chr(code)
      paths = tuple( _path_to_line(p.get("d")) for p in e.findall(".//path") )
      if not (0x4e00 <= code <= 0x9fff): continue
      kanji = data.setdefault(len(paths), {})
      assert char not in kanji
      kanji[char] = paths
  return data
                                                                # }}}1

# https://www.w3.org/TR/SVG/paths.html
def _path_to_line(path):                                        # {{{1
  assert path[0] in "Mm"
  last = 0
  for i, m in enumerate(PATH_RX.finditer(path)):
    assert m.start() == last
    last = m.end()
    cmd, args = m.group(1), tuple(map(float, ARGS_RX.findall(m.group(2))))
    if cmd in "Mm":   # moveto
      assert i == 0
      x1, y1 = x2, y2 = args
    elif cmd in "Zz": # closepath
      assert len(args) == 0
      assert last == len(path)
      x2, y2 = x1, y1
    elif cmd in "Cc": # curveto
      while args:
        _x1, _y1, _x2, _y2, x, y, *args = args
        x2, y2 = (x, y) if cmd.isupper() else (x2 + x, y2 + y)
    elif cmd in "Ss": # shorthand/smooth curveto
      _x2, _y2, x, y = args
      x2, y2 = (x, y) if cmd.isupper() else (x2 + x, y2 + y)
    else:
      assert False
  assert last == len(path)
  assert all( 0 <= v < 109 for v in [x1, y1, x2, y2] )
  return tuple( int(v * 255 / 109) for v in [x1, y1, x2, y2] )
                                                                # }}}1

def _load_json(file = DATAFILE):
  """Load data from JSON file."""
  with open(file) as fh:
    return { int(n): { k: Kanji(v) for k, v in x.items() }
             for n, x in json.load(fh).items() }

def _save_json(file, data):
  """Save data to JSON file."""
  with open(file, "w") as fh:
    json.dump(data, fh, sort_keys = True)

if __name__ == "__main__":
  if "--doctest" in sys.argv:
    verbose = "--verbose" in sys.argv
    import doctest
    if doctest.testmod(verbose = verbose)[0]: sys.exit(1)

# vim: set tw=70 sw=2 sts=2 et fdm=marker :



from pathlib import Path
import setuptools

from kanjidraw import __version__

info = Path(__file__).with_name("README.md").read_text(encoding = "utf8")

setuptools.setup(
  name              = "kanjidraw",
  url               = "https://github.com/obfusk/kanjidraw",
  description       = "handwritten kanji recognition",
  long_description  = info,
  long_description_content_type = "text/markdown",
  version           = __version__,
  author            = "Felix C. Stegerman",
  author_email      = "flx@obfusk.net",
  license           = "AGPLv3+",
  classifiers       = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "Intended Audience :: End Users/Desktop",
    "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
    "Natural Language :: Japanese",
    "Operating System :: POSIX :: Linux",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
  # "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: Implementation :: CPython",
  # "Programming Language :: Python :: Implementation :: PyPy",
    "Topic :: Text Processing",
    "Topic :: Utilities",
  ],
  keywords          = "japanese kanji draw handwriting",
  packages          = setuptools.find_packages(),
  package_data      = dict(kanjidraw = ["data.json"]),
  entry_points      = dict(
    console_scripts = ["kanjidraw = kanjidraw.gui:main"]
  ),
  python_requires   = ">=3.5",
)
